# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Mail.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_Collections_ice
import omero_System_ice
import omero_cmd_API_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')

# Start of module omero
__name__ = 'omero'

# Start of module omero.cmd
__name__ = 'omero.cmd'

if 'SendEmailRequest' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.SendEmailRequest = Ice.createTempClass()
    class SendEmailRequest(_M_omero.cmd.Request):
        """
        Requests an email to be send to all users of the omero
        determines inactive users, an active members of given groups
        and/or specific users.
        examples:
        - omero.cmd.SendEmailRequest(subject, body, everyone=True)
        sends message to everyone who has email set
        and is an active user
        - omero.cmd.SendEmailRequest(subject, body, everyone=True, inactive=True)
        sends message to everyone who has email set,
        even inactive users
        - omero.cmd.SendEmailRequest(subject, body, groupIds=\[...],
        userIds=\[...] )
        sends email to active members of given groups and selected users
        - extra=\[...] allows to set extra email address if not in DB
        """
        def __init__(self, subject='', body='', html=False, userIds=None, groupIds=None, extra=None, inactive=False, everyone=False):
            _M_omero.cmd.Request.__init__(self)
            self.subject = subject
            self.body = body
            self.html = html
            self.userIds = userIds
            self.groupIds = groupIds
            self.extra = extra
            self.inactive = inactive
            self.everyone = everyone

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request', '::omero::cmd::SendEmailRequest')

        def ice_id(self, current=None):
            return '::omero::cmd::SendEmailRequest'

        def ice_staticId():
            return '::omero::cmd::SendEmailRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_SendEmailRequest)

        __repr__ = __str__

    _M_omero.cmd.SendEmailRequestPrx = Ice.createTempClass()
    class SendEmailRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.SendEmailRequestPrx.ice_checkedCast(proxy, '::omero::cmd::SendEmailRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.SendEmailRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::SendEmailRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_SendEmailRequestPrx = IcePy.defineProxy('::omero::cmd::SendEmailRequest', SendEmailRequestPrx)

    _M_omero.cmd._t_SendEmailRequest = IcePy.defineClass('::omero::cmd::SendEmailRequest', SendEmailRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), (
        ('subject', (), IcePy._t_string, False, 0),
        ('body', (), IcePy._t_string, False, 0),
        ('html', (), IcePy._t_bool, False, 0),
        ('userIds', (), _M_omero.sys._t_LongList, False, 0),
        ('groupIds', (), _M_omero.sys._t_LongList, False, 0),
        ('extra', (), _M_omero.api._t_StringSet, False, 0),
        ('inactive', (), IcePy._t_bool, False, 0),
        ('everyone', (), IcePy._t_bool, False, 0)
    ))
    SendEmailRequest._ice_type = _M_omero.cmd._t_SendEmailRequest

    _M_omero.cmd.SendEmailRequest = SendEmailRequest
    del SendEmailRequest

    _M_omero.cmd.SendEmailRequestPrx = SendEmailRequestPrx
    del SendEmailRequestPrx

if 'SendEmailResponse' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.SendEmailResponse = Ice.createTempClass()
    class SendEmailResponse(_M_omero.cmd.Response):
        """
        Successful response for {@code SendEmailRequest}. Contains
        a list of invalid users that has no email address set.
        If no recipients or invalid users found, an {@code ERR} will be
        returned.
        - invalidusers is a list of userIds that email didn't pass criteria
        such as was empty or less then 5 characters
        - invalidemails is a list of email addresses that send email failed
        - total is a total number of email in the pull to be sent.
        - success is a number of emails that were sent successfully.
        """
        def __init__(self, total=0, success=0, invalidusers=None, invalidemails=None):
            _M_omero.cmd.Response.__init__(self)
            self.total = total
            self.success = success
            self.invalidusers = invalidusers
            self.invalidemails = invalidemails

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Response', '::omero::cmd::SendEmailResponse')

        def ice_id(self, current=None):
            return '::omero::cmd::SendEmailResponse'

        def ice_staticId():
            return '::omero::cmd::SendEmailResponse'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_SendEmailResponse)

        __repr__ = __str__

    _M_omero.cmd.SendEmailResponsePrx = Ice.createTempClass()
    class SendEmailResponsePrx(_M_omero.cmd.ResponsePrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.SendEmailResponsePrx.ice_checkedCast(proxy, '::omero::cmd::SendEmailResponse', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.SendEmailResponsePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::SendEmailResponse'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_SendEmailResponsePrx = IcePy.defineProxy('::omero::cmd::SendEmailResponse', SendEmailResponsePrx)

    _M_omero.cmd._t_SendEmailResponse = IcePy.defineClass('::omero::cmd::SendEmailResponse', SendEmailResponse, -1, (), False, False, _M_omero.cmd._t_Response, (), (
        ('total', (), IcePy._t_long, False, 0),
        ('success', (), IcePy._t_long, False, 0),
        ('invalidusers', (), _M_omero.api._t_LongList, False, 0),
        ('invalidemails', (), _M_omero.api._t_StringSet, False, 0)
    ))
    SendEmailResponse._ice_type = _M_omero.cmd._t_SendEmailResponse

    _M_omero.cmd.SendEmailResponse = SendEmailResponse
    del SendEmailResponse

    _M_omero.cmd.SendEmailResponsePrx = SendEmailResponsePrx
    del SendEmailResponsePrx

# End of module omero.cmd

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `ObjectiveSettings.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Medium' not in _M_omero.model.__dict__:
    _M_omero.model._t_Medium = IcePy.declareClass('::omero::model::Medium')
    _M_omero.model._t_MediumPrx = IcePy.declareProxy('::omero::model::Medium')

if 'Objective' not in _M_omero.model.__dict__:
    _M_omero.model._t_Objective = IcePy.declareClass('::omero::model::Objective')
    _M_omero.model._t_ObjectivePrx = IcePy.declareProxy('::omero::model::Objective')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'ObjectiveSettings' not in _M_omero.model.__dict__:
    _M_omero.model.ObjectiveSettings = Ice.createTempClass()
    class ObjectiveSettings(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _correctionCollar=None, _medium=None, _refractiveIndex=None, _objective=None):
            if Ice.getType(self) == _M_omero.model.ObjectiveSettings:
                raise RuntimeError('omero.model.ObjectiveSettings is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._correctionCollar = _correctionCollar
            self._medium = _medium
            self._refractiveIndex = _refractiveIndex
            self._objective = _objective

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::ObjectiveSettings')

        def ice_id(self, current=None):
            return '::omero::model::ObjectiveSettings'

        def ice_staticId():
            return '::omero::model::ObjectiveSettings'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getCorrectionCollar(self, current=None):
            pass

        def setCorrectionCollar(self, theCorrectionCollar, current=None):
            pass

        def getMedium(self, current=None):
            pass

        def setMedium(self, theMedium, current=None):
            pass

        def getRefractiveIndex(self, current=None):
            pass

        def setRefractiveIndex(self, theRefractiveIndex, current=None):
            pass

        def getObjective(self, current=None):
            pass

        def setObjective(self, theObjective, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_ObjectiveSettings)

        __repr__ = __str__

    _M_omero.model.ObjectiveSettingsPrx = Ice.createTempClass()
    class ObjectiveSettingsPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.ObjectiveSettings._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.ObjectiveSettings._op_setVersion.end(self, _r)

        def getCorrectionCollar(self, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getCorrectionCollar.invoke(self, ((), _ctx))

        def begin_getCorrectionCollar(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getCorrectionCollar.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getCorrectionCollar(self, _r):
            return _M_omero.model.ObjectiveSettings._op_getCorrectionCollar.end(self, _r)

        def setCorrectionCollar(self, theCorrectionCollar, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setCorrectionCollar.invoke(self, ((theCorrectionCollar, ), _ctx))

        def begin_setCorrectionCollar(self, theCorrectionCollar, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setCorrectionCollar.begin(self, ((theCorrectionCollar, ), _response, _ex, _sent, _ctx))

        def end_setCorrectionCollar(self, _r):
            return _M_omero.model.ObjectiveSettings._op_setCorrectionCollar.end(self, _r)

        def getMedium(self, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getMedium.invoke(self, ((), _ctx))

        def begin_getMedium(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getMedium.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMedium(self, _r):
            return _M_omero.model.ObjectiveSettings._op_getMedium.end(self, _r)

        def setMedium(self, theMedium, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setMedium.invoke(self, ((theMedium, ), _ctx))

        def begin_setMedium(self, theMedium, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setMedium.begin(self, ((theMedium, ), _response, _ex, _sent, _ctx))

        def end_setMedium(self, _r):
            return _M_omero.model.ObjectiveSettings._op_setMedium.end(self, _r)

        def getRefractiveIndex(self, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getRefractiveIndex.invoke(self, ((), _ctx))

        def begin_getRefractiveIndex(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getRefractiveIndex.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRefractiveIndex(self, _r):
            return _M_omero.model.ObjectiveSettings._op_getRefractiveIndex.end(self, _r)

        def setRefractiveIndex(self, theRefractiveIndex, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setRefractiveIndex.invoke(self, ((theRefractiveIndex, ), _ctx))

        def begin_setRefractiveIndex(self, theRefractiveIndex, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setRefractiveIndex.begin(self, ((theRefractiveIndex, ), _response, _ex, _sent, _ctx))

        def end_setRefractiveIndex(self, _r):
            return _M_omero.model.ObjectiveSettings._op_setRefractiveIndex.end(self, _r)

        def getObjective(self, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getObjective.invoke(self, ((), _ctx))

        def begin_getObjective(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_getObjective.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getObjective(self, _r):
            return _M_omero.model.ObjectiveSettings._op_getObjective.end(self, _r)

        def setObjective(self, theObjective, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setObjective.invoke(self, ((theObjective, ), _ctx))

        def begin_setObjective(self, theObjective, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ObjectiveSettings._op_setObjective.begin(self, ((theObjective, ), _response, _ex, _sent, _ctx))

        def end_setObjective(self, _r):
            return _M_omero.model.ObjectiveSettings._op_setObjective.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.ObjectiveSettingsPrx.ice_checkedCast(proxy, '::omero::model::ObjectiveSettings', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.ObjectiveSettingsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::ObjectiveSettings'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_ObjectiveSettingsPrx = IcePy.defineProxy('::omero::model::ObjectiveSettings', ObjectiveSettingsPrx)

    _M_omero.model._t_ObjectiveSettings = IcePy.declareClass('::omero::model::ObjectiveSettings')

    _M_omero.model._t_ObjectiveSettings = IcePy.defineClass('::omero::model::ObjectiveSettings', ObjectiveSettings, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_correctionCollar', (), _M_omero._t_RDouble, False, 0),
        ('_medium', (), _M_omero.model._t_Medium, False, 0),
        ('_refractiveIndex', (), _M_omero._t_RDouble, False, 0),
        ('_objective', (), _M_omero.model._t_Objective, False, 0)
    ))
    ObjectiveSettings._ice_type = _M_omero.model._t_ObjectiveSettings

    ObjectiveSettings._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    ObjectiveSettings._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    ObjectiveSettings._op_getCorrectionCollar = IcePy.Operation('getCorrectionCollar', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    ObjectiveSettings._op_setCorrectionCollar = IcePy.Operation('setCorrectionCollar', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    ObjectiveSettings._op_getMedium = IcePy.Operation('getMedium', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Medium, False, 0), ())
    ObjectiveSettings._op_setMedium = IcePy.Operation('setMedium', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Medium, False, 0),), (), None, ())
    ObjectiveSettings._op_getRefractiveIndex = IcePy.Operation('getRefractiveIndex', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    ObjectiveSettings._op_setRefractiveIndex = IcePy.Operation('setRefractiveIndex', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    ObjectiveSettings._op_getObjective = IcePy.Operation('getObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Objective, False, 0), ())
    ObjectiveSettings._op_setObjective = IcePy.Operation('setObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Objective, False, 0),), (), None, ())

    _M_omero.model.ObjectiveSettings = ObjectiveSettings
    del ObjectiveSettings

    _M_omero.model.ObjectiveSettingsPrx = ObjectiveSettingsPrx
    del ObjectiveSettingsPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# 
# Copyright 2009 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt

"""
Library for gateway tests
"""
from __future__ import print_function

from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import range
from builtins import object
from io import StringIO
from io import BytesIO
import omero
from omero.rtypes import rstring

from omero.gateway.scripts import dbhelpers

dbhelpers.USERS = {
    'user': dbhelpers.UserEntry(
        'weblitz_test_user', 'foobar', 'User', 'Weblitz'),
    'author': dbhelpers.UserEntry(
        'weblitz_test_author', 'foobar', 'Author', 'Weblitz'),
}

dbhelpers.PROJECTS = {
    'testpr1': dbhelpers.ProjectEntry('weblitz_test_priv_project', 'author'),
    'testpr2': dbhelpers.ProjectEntry('weblitz_test_priv_project2', 'author'),
}

dbhelpers.DATASETS = {
    'testds1': dbhelpers.DatasetEntry('weblitz_test_priv_dataset', 'testpr1'),
    'testds2': dbhelpers.DatasetEntry('weblitz_test_priv_dataset2', 'testpr1'),
    'testds3': dbhelpers.DatasetEntry('weblitz_test_priv_dataset3', 'testpr2'),
}

dbhelpers.IMAGES = {
    'testimg1': dbhelpers.ImageEntry(
        'weblitz_test_priv_image', 'CHOBI_d3d.dv', 'testds1'),
    'testimg2': dbhelpers.ImageEntry(
        'weblitz_test_priv_image2', 'CHOBI_d3d.dv', 'testds1'),
    'tinyimg': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_tiny', 'tinyTest.d3d.dv', 'testds1'),
    'badimg': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_bad', False, 'testds1'),
    'tinyimg2': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_tiny2', 'tinyTest.d3d.dv', 'testds2'),
    'tinyimg3': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_tiny3', 'tinyTest.d3d.dv', 'testds3'),
    'bigimg': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_big', 'big.tiff', 'testds3'),
    '32float': dbhelpers.ImageEntry(
        'weblitz_test_priv_image_32float',
        '32bitfloat&pixelType=float&sizeX=8192&sizeY=8192.fake', 'testds3'),
}


class TestDBHelper(object):

    def setUp(self, skipTestDB=False, skipTestImages=True):
        self.tmpfiles = []
        self._has_connected = False
        self._last_login = None
        self.doDisconnect()
        self.USER = dbhelpers.USERS['user']
        self.AUTHOR = dbhelpers.USERS['author']
        self.ADMIN = dbhelpers.ROOT
        gateway = omero.client_wrapper()
        try:
            rp = gateway.getProperty('omero.rootpass')
            if rp:
                dbhelpers.ROOT.passwd = rp
        finally:
            gateway.close()

        self.prepTestDB(onlyUsers=skipTestDB, skipImages=skipTestImages)
        self.doDisconnect()

    def doConnect(self):
        if not self._has_connected:
            self.gateway.connect()
            self._has_connected = True
        assert self.gateway.isConnected(), 'Can not connect'
        assert self.gateway.keepAlive(
        ), 'Could not send keepAlive to connection'
        self.gateway.setGroupForSession(
            self.gateway.getEventContext().memberOfGroups[0])

    def doDisconnect(self):
        if self._has_connected and self.gateway:
            self.doConnect()
            self.gateway.close()
            assert not self.gateway.isConnected(), 'Can not disconnect'
        self.gateway = None
        self._has_connected = False
        self._last_login = None

    def doLogin(self, user=None, groupname=None):
        l = (user, groupname)
        if self._has_connected and self._last_login == l:
            return self.doConnect()
        self.doDisconnect()
        if user:
            self.gateway = dbhelpers.login(user, groupname)
        else:
            self.gateway = dbhelpers.loginAsPublic()
        self.doConnect()
        self._last_login = l

    def loginAsAdmin(self):
        self.doLogin(self.ADMIN)

    def loginAsAuthor(self):
        self.doLogin(self.AUTHOR)

    def loginAsUser(self):
        self.doLogin(self.USER)

    def loginAsPublic(self):
        self.doLogin()

    def tearDown(self):

        try:
            if self.gateway is not None:
                self.gateway.close()
        finally:
            failure = False
            for tmpfile in self.tmpfiles:
                try:
                    tmpfile.close()
                except:
                    print("Error closing:" + tmpfile)
        if failure:
            raise Exception("Exception on client.closeSession")

    def getTestProject(self):
        return dbhelpers.getProject(self.gateway, 'testpr1')

    def getTestProject2(self):
        return dbhelpers.getProject(self.gateway, 'testpr2')

    def getTestDataset(self, project=None):
        return dbhelpers.getDataset(self.gateway, 'testds1', project)

    def getTestDataset2(self, project=None):
        return dbhelpers.getDataset(self.gateway, 'testds2', project)

    def getTestImage(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'testimg1', forceds=dataset,
                                  autocreate=autocreate)

    def getTestImage2(self, dataset=None):
        return dbhelpers.getImage(self.gateway, 'testimg2', dataset)

    def getBadTestImage(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'badimg', forceds=dataset,
                                  autocreate=autocreate)

    def getTinyTestImage(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'tinyimg', forceds=dataset,
                                  autocreate=autocreate)

    def getTinyTestImage2(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'tinyimg2', forceds=dataset,
                                  autocreate=autocreate)

    def getTinyTestImage3(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'tinyimg3', forceds=dataset,
                                  autocreate=autocreate)

    def getBigTestImage(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, 'bigimg', forceds=dataset,
                                  autocreate=autocreate)

    def get32FloatTestImage(self, dataset=None, autocreate=False):
        return dbhelpers.getImage(self.gateway, '32float', forceds=dataset,
                                  autocreate=autocreate)

    def prepTestDB(self, onlyUsers=False, skipImages=True):
        dbhelpers.bootstrap(onlyUsers=onlyUsers, skipImages=skipImages)

    def waitOnCmd(self, client, handle):
        callback = omero.callbacks.CmdCallbackI(client, handle)
        callback.loop(10, 500)  # throws on timeout
        rsp = callback.getResponse()
        assert isinstance(rsp, omero.cmd.OK)
        return callback

    def createPDTree(self, project=None, dataset=None):
        """
        Create/link a Project and/or Dataset (link them if both are specified)
        Existing objects can be parsed as an omero.model object(s) or blitz
        Wrapper objects. Otherwise new objects will be created with name
        str(project) or str(dataset). If project OR dataset is specified, the
        ProjectWrapper or DatasetWrapper is returned. If both project and
        dataset are specified, they will be linked and the PD-link is returned
        as a BlitzObjectWrapper.

        @param project:     omero.model.ProjectDatasetLinkI
                            OR omero.gateway.ProjectWrapper
                            or name (string)
        @param dataset:     omero.model.DatasetI
                            OR omero.gateway.DatasetWrapper
                            or name (string)
        """
        dsId = ds = None
        prId = pr = None
        returnVal = None
        if dataset is not None:
            try:
                dsId = dataset.id
                dsId = dsId.val
            except:
                ds = omero.model.DatasetI()
                ds.name = rstring(str(dataset))
                ds = self.gateway.getUpdateService().saveAndReturnObject(ds)
                returnVal = omero.gateway.DatasetWrapper(self.gateway, ds)
                dsId = ds.id.val
        if project is not None:
            try:
                prId = project.id
                prId = prId.val
            except:
                pr = omero.model.ProjectI()
                pr.name = rstring(str(project))
                pr = self.gateway.getUpdateService().saveAndReturnObject(pr)
                returnVal = omero.gateway.ProjectWrapper(self.gateway, pr)
                prId = pr.id.val
        if dsId and prId:
            link = omero.model.ProjectDatasetLinkI()
            link.setParent(omero.model.ProjectI(prId, False))
            link.setChild(omero.model.DatasetI(dsId, False))
            link = self.gateway.getUpdateService().saveAndReturnObject(link)
            returnVal = omero.gateway.BlitzObjectWrapper(self.gateway, link)

        return returnVal

    def createTestImage(self, imageName="testImage", dataset=None, sizeX=16,
                        sizeY=16, sizeZ=1, sizeC=1, sizeT=1):
        """
        Creates a test image of the required dimensions, where each pixel
        value is set to the average value of x & y. If dataset (obj or name)
        is specified, will be linked to image. If project (obj or name) is
        specified, will be created/linked to dataset (if dataset not None)

        @param dataset:     omero.model.DatasetI
                            OR DatasetWrapper
                            OR dataset ID
        """
        from numpy import fromfunction, int16

        def f(x, y):
            return x

        def planeGen():
            for p in range(sizeZ * sizeC * sizeT):
                yield fromfunction(f, (sizeY, sizeX), dtype=int16)

        ds = None
        if dataset is not None:
            if hasattr(dataset, "_obj"):
                dataset = dataset._obj
            if isinstance(dataset, omero.model.DatasetI):
                ds = dataset
            else:
                try:
                    dsId = int(dataset)
                    ds = omero.model.DatasetI(dsId, False)
                except:
                    pass

        image = self.gateway.createImageFromNumpySeq(
            planeGen(), imageName, sizeZ=sizeZ, sizeC=sizeC, sizeT=sizeT,
            dataset=ds)
        return image

    def createTestFile(self, parentpath, filename, content):
        """
        Creates an OriginalFile with the supplied content

        @param parentpath:  Parent directory of the file
        @param filename:    Filename
        @param content:     String containing the content of the file
        @return:            OriginalFileWrapper
        """
        sio = BytesIO(content.encode("utf-8"))
        f = self.gateway.createOriginalFileFromFileObj(
            sio, parentpath, filename, len(content))
        return f



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `FilterSet.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Instrument' not in _M_omero.model.__dict__:
    _M_omero.model._t_Instrument = IcePy.declareClass('::omero::model::Instrument')
    _M_omero.model._t_InstrumentPrx = IcePy.declareProxy('::omero::model::Instrument')

if 'FilterSetExcitationFilterLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetExcitationFilterLink = IcePy.declareClass('::omero::model::FilterSetExcitationFilterLink')
    _M_omero.model._t_FilterSetExcitationFilterLinkPrx = IcePy.declareProxy('::omero::model::FilterSetExcitationFilterLink')

if 'Filter' not in _M_omero.model.__dict__:
    _M_omero.model._t_Filter = IcePy.declareClass('::omero::model::Filter')
    _M_omero.model._t_FilterPrx = IcePy.declareProxy('::omero::model::Filter')

if 'Dichroic' not in _M_omero.model.__dict__:
    _M_omero.model._t_Dichroic = IcePy.declareClass('::omero::model::Dichroic')
    _M_omero.model._t_DichroicPrx = IcePy.declareProxy('::omero::model::Dichroic')

if 'FilterSetEmissionFilterLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetEmissionFilterLink = IcePy.declareClass('::omero::model::FilterSetEmissionFilterLink')
    _M_omero.model._t_FilterSetEmissionFilterLinkPrx = IcePy.declareProxy('::omero::model::FilterSetEmissionFilterLink')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_FilterSetExcitationFilterLinkSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetExcitationFilterLinkSeq = IcePy.defineSequence('::omero::model::FilterSetExcitationFilterLinkSeq', (), _M_omero.model._t_FilterSetExcitationFilterLink)

if '_t_FilterSetLinkedExcitationFilterSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetLinkedExcitationFilterSeq = IcePy.defineSequence('::omero::model::FilterSetLinkedExcitationFilterSeq', (), _M_omero.model._t_Filter)

if '_t_FilterSetEmissionFilterLinkSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetEmissionFilterLinkSeq = IcePy.defineSequence('::omero::model::FilterSetEmissionFilterLinkSeq', (), _M_omero.model._t_FilterSetEmissionFilterLink)

if '_t_FilterSetLinkedEmissionFilterSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSetLinkedEmissionFilterSeq = IcePy.defineSequence('::omero::model::FilterSetLinkedEmissionFilterSeq', (), _M_omero.model._t_Filter)

if 'FilterSet' not in _M_omero.model.__dict__:
    _M_omero.model.FilterSet = Ice.createTempClass()
    class FilterSet(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _manufacturer=None, _model=None, _lotNumber=None, _serialNumber=None, _instrument=None, _excitationFilterLinkSeq=None, _excitationFilterLinkLoaded=False, _excitationFilterLinkCountPerOwner=None, _dichroic=None, _emissionFilterLinkSeq=None, _emissionFilterLinkLoaded=False, _emissionFilterLinkCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.FilterSet:
                raise RuntimeError('omero.model.FilterSet is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._manufacturer = _manufacturer
            self._model = _model
            self._lotNumber = _lotNumber
            self._serialNumber = _serialNumber
            self._instrument = _instrument
            self._excitationFilterLinkSeq = _excitationFilterLinkSeq
            self._excitationFilterLinkLoaded = _excitationFilterLinkLoaded
            self._excitationFilterLinkCountPerOwner = _excitationFilterLinkCountPerOwner
            self._dichroic = _dichroic
            self._emissionFilterLinkSeq = _emissionFilterLinkSeq
            self._emissionFilterLinkLoaded = _emissionFilterLinkLoaded
            self._emissionFilterLinkCountPerOwner = _emissionFilterLinkCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::FilterSet', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::FilterSet'

        def ice_staticId():
            return '::omero::model::FilterSet'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getManufacturer(self, current=None):
            pass

        def setManufacturer(self, theManufacturer, current=None):
            pass

        def getModel(self, current=None):
            pass

        def setModel(self, theModel, current=None):
            pass

        def getLotNumber(self, current=None):
            pass

        def setLotNumber(self, theLotNumber, current=None):
            pass

        def getSerialNumber(self, current=None):
            pass

        def setSerialNumber(self, theSerialNumber, current=None):
            pass

        def getInstrument(self, current=None):
            pass

        def setInstrument(self, theInstrument, current=None):
            pass

        def unloadExcitationFilterLink(self, current=None):
            pass

        def sizeOfExcitationFilterLink(self, current=None):
            pass

        def copyExcitationFilterLink(self, current=None):
            pass

        def addFilterSetExcitationFilterLink(self, target, current=None):
            pass

        def addAllFilterSetExcitationFilterLinkSet(self, targets, current=None):
            pass

        def removeFilterSetExcitationFilterLink(self, theTarget, current=None):
            pass

        def removeAllFilterSetExcitationFilterLinkSet(self, targets, current=None):
            pass

        def clearExcitationFilterLink(self, current=None):
            pass

        def reloadExcitationFilterLink(self, toCopy, current=None):
            pass

        def getExcitationFilterLinkCountPerOwner(self, current=None):
            pass

        def linkExcitationFilter(self, addition, current=None):
            pass

        def addFilterSetExcitationFilterLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFilterSetExcitationFilterLink(self, removal, current=None):
            pass

        def unlinkExcitationFilter(self, removal, current=None):
            pass

        def removeFilterSetExcitationFilterLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedExcitationFilterList(self, current=None):
            pass

        def getDichroic(self, current=None):
            pass

        def setDichroic(self, theDichroic, current=None):
            pass

        def unloadEmissionFilterLink(self, current=None):
            pass

        def sizeOfEmissionFilterLink(self, current=None):
            pass

        def copyEmissionFilterLink(self, current=None):
            pass

        def addFilterSetEmissionFilterLink(self, target, current=None):
            pass

        def addAllFilterSetEmissionFilterLinkSet(self, targets, current=None):
            pass

        def removeFilterSetEmissionFilterLink(self, theTarget, current=None):
            pass

        def removeAllFilterSetEmissionFilterLinkSet(self, targets, current=None):
            pass

        def clearEmissionFilterLink(self, current=None):
            pass

        def reloadEmissionFilterLink(self, toCopy, current=None):
            pass

        def getEmissionFilterLinkCountPerOwner(self, current=None):
            pass

        def linkEmissionFilter(self, addition, current=None):
            pass

        def addFilterSetEmissionFilterLinkToBoth(self, link, bothSides, current=None):
            pass

        def findFilterSetEmissionFilterLink(self, removal, current=None):
            pass

        def unlinkEmissionFilter(self, removal, current=None):
            pass

        def removeFilterSetEmissionFilterLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedEmissionFilterList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_FilterSet)

        __repr__ = __str__

    _M_omero.model.FilterSetPrx = Ice.createTempClass()
    class FilterSetPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.FilterSet._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.FilterSet._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.FilterSet._op_setVersion.end(self, _r)

        def getManufacturer(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getManufacturer.invoke(self, ((), _ctx))

        def begin_getManufacturer(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getManufacturer.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getManufacturer(self, _r):
            return _M_omero.model.FilterSet._op_getManufacturer.end(self, _r)

        def setManufacturer(self, theManufacturer, _ctx=None):
            return _M_omero.model.FilterSet._op_setManufacturer.invoke(self, ((theManufacturer, ), _ctx))

        def begin_setManufacturer(self, theManufacturer, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setManufacturer.begin(self, ((theManufacturer, ), _response, _ex, _sent, _ctx))

        def end_setManufacturer(self, _r):
            return _M_omero.model.FilterSet._op_setManufacturer.end(self, _r)

        def getModel(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getModel.invoke(self, ((), _ctx))

        def begin_getModel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getModel.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getModel(self, _r):
            return _M_omero.model.FilterSet._op_getModel.end(self, _r)

        def setModel(self, theModel, _ctx=None):
            return _M_omero.model.FilterSet._op_setModel.invoke(self, ((theModel, ), _ctx))

        def begin_setModel(self, theModel, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setModel.begin(self, ((theModel, ), _response, _ex, _sent, _ctx))

        def end_setModel(self, _r):
            return _M_omero.model.FilterSet._op_setModel.end(self, _r)

        def getLotNumber(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getLotNumber.invoke(self, ((), _ctx))

        def begin_getLotNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getLotNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLotNumber(self, _r):
            return _M_omero.model.FilterSet._op_getLotNumber.end(self, _r)

        def setLotNumber(self, theLotNumber, _ctx=None):
            return _M_omero.model.FilterSet._op_setLotNumber.invoke(self, ((theLotNumber, ), _ctx))

        def begin_setLotNumber(self, theLotNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setLotNumber.begin(self, ((theLotNumber, ), _response, _ex, _sent, _ctx))

        def end_setLotNumber(self, _r):
            return _M_omero.model.FilterSet._op_setLotNumber.end(self, _r)

        def getSerialNumber(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getSerialNumber.invoke(self, ((), _ctx))

        def begin_getSerialNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getSerialNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSerialNumber(self, _r):
            return _M_omero.model.FilterSet._op_getSerialNumber.end(self, _r)

        def setSerialNumber(self, theSerialNumber, _ctx=None):
            return _M_omero.model.FilterSet._op_setSerialNumber.invoke(self, ((theSerialNumber, ), _ctx))

        def begin_setSerialNumber(self, theSerialNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setSerialNumber.begin(self, ((theSerialNumber, ), _response, _ex, _sent, _ctx))

        def end_setSerialNumber(self, _r):
            return _M_omero.model.FilterSet._op_setSerialNumber.end(self, _r)

        def getInstrument(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getInstrument.invoke(self, ((), _ctx))

        def begin_getInstrument(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getInstrument.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getInstrument(self, _r):
            return _M_omero.model.FilterSet._op_getInstrument.end(self, _r)

        def setInstrument(self, theInstrument, _ctx=None):
            return _M_omero.model.FilterSet._op_setInstrument.invoke(self, ((theInstrument, ), _ctx))

        def begin_setInstrument(self, theInstrument, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setInstrument.begin(self, ((theInstrument, ), _response, _ex, _sent, _ctx))

        def end_setInstrument(self, _r):
            return _M_omero.model.FilterSet._op_setInstrument.end(self, _r)

        def unloadExcitationFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_unloadExcitationFilterLink.invoke(self, ((), _ctx))

        def begin_unloadExcitationFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_unloadExcitationFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_unloadExcitationFilterLink.end(self, _r)

        def sizeOfExcitationFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_sizeOfExcitationFilterLink.invoke(self, ((), _ctx))

        def begin_sizeOfExcitationFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_sizeOfExcitationFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_sizeOfExcitationFilterLink.end(self, _r)

        def copyExcitationFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_copyExcitationFilterLink.invoke(self, ((), _ctx))

        def begin_copyExcitationFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_copyExcitationFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_copyExcitationFilterLink.end(self, _r)

        def addFilterSetExcitationFilterLink(self, target, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLink.invoke(self, ((target, ), _ctx))

        def begin_addFilterSetExcitationFilterLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilterSetExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLink.end(self, _r)

        def addAllFilterSetExcitationFilterLinkSet(self, targets, _ctx=None):
            return _M_omero.model.FilterSet._op_addAllFilterSetExcitationFilterLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilterSetExcitationFilterLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addAllFilterSetExcitationFilterLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilterSetExcitationFilterLinkSet(self, _r):
            return _M_omero.model.FilterSet._op_addAllFilterSetExcitationFilterLinkSet.end(self, _r)

        def removeFilterSetExcitationFilterLink(self, theTarget, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilterSetExcitationFilterLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilterSetExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLink.end(self, _r)

        def removeAllFilterSetExcitationFilterLinkSet(self, targets, _ctx=None):
            return _M_omero.model.FilterSet._op_removeAllFilterSetExcitationFilterLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilterSetExcitationFilterLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeAllFilterSetExcitationFilterLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilterSetExcitationFilterLinkSet(self, _r):
            return _M_omero.model.FilterSet._op_removeAllFilterSetExcitationFilterLinkSet.end(self, _r)

        def clearExcitationFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_clearExcitationFilterLink.invoke(self, ((), _ctx))

        def begin_clearExcitationFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_clearExcitationFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_clearExcitationFilterLink.end(self, _r)

        def reloadExcitationFilterLink(self, toCopy, _ctx=None):
            return _M_omero.model.FilterSet._op_reloadExcitationFilterLink.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadExcitationFilterLink(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_reloadExcitationFilterLink.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_reloadExcitationFilterLink.end(self, _r)

        def getExcitationFilterLinkCountPerOwner(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getExcitationFilterLinkCountPerOwner.invoke(self, ((), _ctx))

        def begin_getExcitationFilterLinkCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getExcitationFilterLinkCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getExcitationFilterLinkCountPerOwner(self, _r):
            return _M_omero.model.FilterSet._op_getExcitationFilterLinkCountPerOwner.end(self, _r)

        def linkExcitationFilter(self, addition, _ctx=None):
            return _M_omero.model.FilterSet._op_linkExcitationFilter.invoke(self, ((addition, ), _ctx))

        def begin_linkExcitationFilter(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_linkExcitationFilter.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkExcitationFilter(self, _r):
            return _M_omero.model.FilterSet._op_linkExcitationFilter.end(self, _r)

        def addFilterSetExcitationFilterLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFilterSetExcitationFilterLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFilterSetExcitationFilterLinkToBoth(self, _r):
            return _M_omero.model.FilterSet._op_addFilterSetExcitationFilterLinkToBoth.end(self, _r)

        def findFilterSetExcitationFilterLink(self, removal, _ctx=None):
            return _M_omero.model.FilterSet._op_findFilterSetExcitationFilterLink.invoke(self, ((removal, ), _ctx))

        def begin_findFilterSetExcitationFilterLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_findFilterSetExcitationFilterLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFilterSetExcitationFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_findFilterSetExcitationFilterLink.end(self, _r)

        def unlinkExcitationFilter(self, removal, _ctx=None):
            return _M_omero.model.FilterSet._op_unlinkExcitationFilter.invoke(self, ((removal, ), _ctx))

        def begin_unlinkExcitationFilter(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_unlinkExcitationFilter.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkExcitationFilter(self, _r):
            return _M_omero.model.FilterSet._op_unlinkExcitationFilter.end(self, _r)

        def removeFilterSetExcitationFilterLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFilterSetExcitationFilterLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFilterSetExcitationFilterLinkFromBoth(self, _r):
            return _M_omero.model.FilterSet._op_removeFilterSetExcitationFilterLinkFromBoth.end(self, _r)

        def linkedExcitationFilterList(self, _ctx=None):
            return _M_omero.model.FilterSet._op_linkedExcitationFilterList.invoke(self, ((), _ctx))

        def begin_linkedExcitationFilterList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_linkedExcitationFilterList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedExcitationFilterList(self, _r):
            return _M_omero.model.FilterSet._op_linkedExcitationFilterList.end(self, _r)

        def getDichroic(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getDichroic.invoke(self, ((), _ctx))

        def begin_getDichroic(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getDichroic.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDichroic(self, _r):
            return _M_omero.model.FilterSet._op_getDichroic.end(self, _r)

        def setDichroic(self, theDichroic, _ctx=None):
            return _M_omero.model.FilterSet._op_setDichroic.invoke(self, ((theDichroic, ), _ctx))

        def begin_setDichroic(self, theDichroic, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_setDichroic.begin(self, ((theDichroic, ), _response, _ex, _sent, _ctx))

        def end_setDichroic(self, _r):
            return _M_omero.model.FilterSet._op_setDichroic.end(self, _r)

        def unloadEmissionFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_unloadEmissionFilterLink.invoke(self, ((), _ctx))

        def begin_unloadEmissionFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_unloadEmissionFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_unloadEmissionFilterLink.end(self, _r)

        def sizeOfEmissionFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_sizeOfEmissionFilterLink.invoke(self, ((), _ctx))

        def begin_sizeOfEmissionFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_sizeOfEmissionFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_sizeOfEmissionFilterLink.end(self, _r)

        def copyEmissionFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_copyEmissionFilterLink.invoke(self, ((), _ctx))

        def begin_copyEmissionFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_copyEmissionFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_copyEmissionFilterLink.end(self, _r)

        def addFilterSetEmissionFilterLink(self, target, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLink.invoke(self, ((target, ), _ctx))

        def begin_addFilterSetEmissionFilterLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilterSetEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLink.end(self, _r)

        def addAllFilterSetEmissionFilterLinkSet(self, targets, _ctx=None):
            return _M_omero.model.FilterSet._op_addAllFilterSetEmissionFilterLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilterSetEmissionFilterLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addAllFilterSetEmissionFilterLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilterSetEmissionFilterLinkSet(self, _r):
            return _M_omero.model.FilterSet._op_addAllFilterSetEmissionFilterLinkSet.end(self, _r)

        def removeFilterSetEmissionFilterLink(self, theTarget, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilterSetEmissionFilterLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilterSetEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLink.end(self, _r)

        def removeAllFilterSetEmissionFilterLinkSet(self, targets, _ctx=None):
            return _M_omero.model.FilterSet._op_removeAllFilterSetEmissionFilterLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilterSetEmissionFilterLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeAllFilterSetEmissionFilterLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilterSetEmissionFilterLinkSet(self, _r):
            return _M_omero.model.FilterSet._op_removeAllFilterSetEmissionFilterLinkSet.end(self, _r)

        def clearEmissionFilterLink(self, _ctx=None):
            return _M_omero.model.FilterSet._op_clearEmissionFilterLink.invoke(self, ((), _ctx))

        def begin_clearEmissionFilterLink(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_clearEmissionFilterLink.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_clearEmissionFilterLink.end(self, _r)

        def reloadEmissionFilterLink(self, toCopy, _ctx=None):
            return _M_omero.model.FilterSet._op_reloadEmissionFilterLink.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadEmissionFilterLink(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_reloadEmissionFilterLink.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_reloadEmissionFilterLink.end(self, _r)

        def getEmissionFilterLinkCountPerOwner(self, _ctx=None):
            return _M_omero.model.FilterSet._op_getEmissionFilterLinkCountPerOwner.invoke(self, ((), _ctx))

        def begin_getEmissionFilterLinkCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_getEmissionFilterLinkCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getEmissionFilterLinkCountPerOwner(self, _r):
            return _M_omero.model.FilterSet._op_getEmissionFilterLinkCountPerOwner.end(self, _r)

        def linkEmissionFilter(self, addition, _ctx=None):
            return _M_omero.model.FilterSet._op_linkEmissionFilter.invoke(self, ((addition, ), _ctx))

        def begin_linkEmissionFilter(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_linkEmissionFilter.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkEmissionFilter(self, _r):
            return _M_omero.model.FilterSet._op_linkEmissionFilter.end(self, _r)

        def addFilterSetEmissionFilterLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addFilterSetEmissionFilterLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addFilterSetEmissionFilterLinkToBoth(self, _r):
            return _M_omero.model.FilterSet._op_addFilterSetEmissionFilterLinkToBoth.end(self, _r)

        def findFilterSetEmissionFilterLink(self, removal, _ctx=None):
            return _M_omero.model.FilterSet._op_findFilterSetEmissionFilterLink.invoke(self, ((removal, ), _ctx))

        def begin_findFilterSetEmissionFilterLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_findFilterSetEmissionFilterLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findFilterSetEmissionFilterLink(self, _r):
            return _M_omero.model.FilterSet._op_findFilterSetEmissionFilterLink.end(self, _r)

        def unlinkEmissionFilter(self, removal, _ctx=None):
            return _M_omero.model.FilterSet._op_unlinkEmissionFilter.invoke(self, ((removal, ), _ctx))

        def begin_unlinkEmissionFilter(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_unlinkEmissionFilter.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkEmissionFilter(self, _r):
            return _M_omero.model.FilterSet._op_unlinkEmissionFilter.end(self, _r)

        def removeFilterSetEmissionFilterLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeFilterSetEmissionFilterLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeFilterSetEmissionFilterLinkFromBoth(self, _r):
            return _M_omero.model.FilterSet._op_removeFilterSetEmissionFilterLinkFromBoth.end(self, _r)

        def linkedEmissionFilterList(self, _ctx=None):
            return _M_omero.model.FilterSet._op_linkedEmissionFilterList.invoke(self, ((), _ctx))

        def begin_linkedEmissionFilterList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.FilterSet._op_linkedEmissionFilterList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedEmissionFilterList(self, _r):
            return _M_omero.model.FilterSet._op_linkedEmissionFilterList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.FilterSetPrx.ice_checkedCast(proxy, '::omero::model::FilterSet', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.FilterSetPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::FilterSet'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_FilterSetPrx = IcePy.defineProxy('::omero::model::FilterSet', FilterSetPrx)

    _M_omero.model._t_FilterSet = IcePy.declareClass('::omero::model::FilterSet')

    _M_omero.model._t_FilterSet = IcePy.defineClass('::omero::model::FilterSet', FilterSet, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_manufacturer', (), _M_omero._t_RString, False, 0),
        ('_model', (), _M_omero._t_RString, False, 0),
        ('_lotNumber', (), _M_omero._t_RString, False, 0),
        ('_serialNumber', (), _M_omero._t_RString, False, 0),
        ('_instrument', (), _M_omero.model._t_Instrument, False, 0),
        ('_excitationFilterLinkSeq', (), _M_omero.model._t_FilterSetExcitationFilterLinkSeq, False, 0),
        ('_excitationFilterLinkLoaded', (), IcePy._t_bool, False, 0),
        ('_excitationFilterLinkCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_dichroic', (), _M_omero.model._t_Dichroic, False, 0),
        ('_emissionFilterLinkSeq', (), _M_omero.model._t_FilterSetEmissionFilterLinkSeq, False, 0),
        ('_emissionFilterLinkLoaded', (), IcePy._t_bool, False, 0),
        ('_emissionFilterLinkCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    FilterSet._ice_type = _M_omero.model._t_FilterSet

    FilterSet._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    FilterSet._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    FilterSet._op_getManufacturer = IcePy.Operation('getManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    FilterSet._op_setManufacturer = IcePy.Operation('setManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    FilterSet._op_getModel = IcePy.Operation('getModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    FilterSet._op_setModel = IcePy.Operation('setModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    FilterSet._op_getLotNumber = IcePy.Operation('getLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    FilterSet._op_setLotNumber = IcePy.Operation('setLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    FilterSet._op_getSerialNumber = IcePy.Operation('getSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    FilterSet._op_setSerialNumber = IcePy.Operation('setSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    FilterSet._op_getInstrument = IcePy.Operation('getInstrument', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Instrument, False, 0), ())
    FilterSet._op_setInstrument = IcePy.Operation('setInstrument', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    FilterSet._op_unloadExcitationFilterLink = IcePy.Operation('unloadExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    FilterSet._op_sizeOfExcitationFilterLink = IcePy.Operation('sizeOfExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    FilterSet._op_copyExcitationFilterLink = IcePy.Operation('copyExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilterSetExcitationFilterLinkSeq, False, 0), ())
    FilterSet._op_addFilterSetExcitationFilterLink = IcePy.Operation('addFilterSetExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLink, False, 0),), (), None, ())
    FilterSet._op_addAllFilterSetExcitationFilterLinkSet = IcePy.Operation('addAllFilterSetExcitationFilterLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLinkSeq, False, 0),), (), None, ())
    FilterSet._op_removeFilterSetExcitationFilterLink = IcePy.Operation('removeFilterSetExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLink, False, 0),), (), None, ())
    FilterSet._op_removeAllFilterSetExcitationFilterLinkSet = IcePy.Operation('removeAllFilterSetExcitationFilterLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLinkSeq, False, 0),), (), None, ())
    FilterSet._op_clearExcitationFilterLink = IcePy.Operation('clearExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    FilterSet._op_reloadExcitationFilterLink = IcePy.Operation('reloadExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSet, False, 0),), (), None, ())
    FilterSet._op_getExcitationFilterLinkCountPerOwner = IcePy.Operation('getExcitationFilterLinkCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    FilterSet._op_linkExcitationFilter = IcePy.Operation('linkExcitationFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), ((), _M_omero.model._t_FilterSetExcitationFilterLink, False, 0), ())
    FilterSet._op_addFilterSetExcitationFilterLinkToBoth = IcePy.Operation('addFilterSetExcitationFilterLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    FilterSet._op_findFilterSetExcitationFilterLink = IcePy.Operation('findFilterSetExcitationFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), ((), _M_omero.model._t_FilterSetExcitationFilterLinkSeq, False, 0), ())
    FilterSet._op_unlinkExcitationFilter = IcePy.Operation('unlinkExcitationFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), None, ())
    FilterSet._op_removeFilterSetExcitationFilterLinkFromBoth = IcePy.Operation('removeFilterSetExcitationFilterLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetExcitationFilterLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    FilterSet._op_linkedExcitationFilterList = IcePy.Operation('linkedExcitationFilterList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilterSetLinkedExcitationFilterSeq, False, 0), ())
    FilterSet._op_getDichroic = IcePy.Operation('getDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Dichroic, False, 0), ())
    FilterSet._op_setDichroic = IcePy.Operation('setDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dichroic, False, 0),), (), None, ())
    FilterSet._op_unloadEmissionFilterLink = IcePy.Operation('unloadEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    FilterSet._op_sizeOfEmissionFilterLink = IcePy.Operation('sizeOfEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    FilterSet._op_copyEmissionFilterLink = IcePy.Operation('copyEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilterSetEmissionFilterLinkSeq, False, 0), ())
    FilterSet._op_addFilterSetEmissionFilterLink = IcePy.Operation('addFilterSetEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLink, False, 0),), (), None, ())
    FilterSet._op_addAllFilterSetEmissionFilterLinkSet = IcePy.Operation('addAllFilterSetEmissionFilterLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLinkSeq, False, 0),), (), None, ())
    FilterSet._op_removeFilterSetEmissionFilterLink = IcePy.Operation('removeFilterSetEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLink, False, 0),), (), None, ())
    FilterSet._op_removeAllFilterSetEmissionFilterLinkSet = IcePy.Operation('removeAllFilterSetEmissionFilterLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLinkSeq, False, 0),), (), None, ())
    FilterSet._op_clearEmissionFilterLink = IcePy.Operation('clearEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    FilterSet._op_reloadEmissionFilterLink = IcePy.Operation('reloadEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSet, False, 0),), (), None, ())
    FilterSet._op_getEmissionFilterLinkCountPerOwner = IcePy.Operation('getEmissionFilterLinkCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    FilterSet._op_linkEmissionFilter = IcePy.Operation('linkEmissionFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), ((), _M_omero.model._t_FilterSetEmissionFilterLink, False, 0), ())
    FilterSet._op_addFilterSetEmissionFilterLinkToBoth = IcePy.Operation('addFilterSetEmissionFilterLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    FilterSet._op_findFilterSetEmissionFilterLink = IcePy.Operation('findFilterSetEmissionFilterLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), ((), _M_omero.model._t_FilterSetEmissionFilterLinkSeq, False, 0), ())
    FilterSet._op_unlinkEmissionFilter = IcePy.Operation('unlinkEmissionFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), None, ())
    FilterSet._op_removeFilterSetEmissionFilterLinkFromBoth = IcePy.Operation('removeFilterSetEmissionFilterLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSetEmissionFilterLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    FilterSet._op_linkedEmissionFilterList = IcePy.Operation('linkedEmissionFilterList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilterSetLinkedEmissionFilterSeq, False, 0), ())

    _M_omero.model.FilterSet = FilterSet
    del FilterSet

    _M_omero.model.FilterSetPrx = FilterSetPrx
    del FilterSetPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `JobHandle.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'JobHandle' not in _M_omero.api.__dict__:
    _M_omero.api.JobHandle = Ice.createTempClass()
    class JobHandle(_M_omero.api.StatefulServiceInterface):
        """
        Allows submission of asynchronous jobs.
        NOTE: The calling order for the service is as follows:
        {@code submit} or {@code attach}
        any of the other methods
        {@code close}
        Calling {@code close} does not cancel or otherwise change
        the Job state. See {@code cancelJob}.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.JobHandle:
                raise RuntimeError('omero.api.JobHandle is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::JobHandle', '::omero::api::ServiceInterface', '::omero::api::StatefulServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::JobHandle'

        def ice_staticId():
            return '::omero::api::JobHandle'
        ice_staticId = staticmethod(ice_staticId)

        def submit_async(self, _cb, job, current=None):
            """
            Submits a omero.model.Job and returns its database
            id. The only fields directly on status which are editable
            are message, scheduledFor and
            status. The latter two must be sensible.
            Arguments:
            _cb -- The asynchronous callback object.
            job -- Not null
            current -- The Current object for the invocation.
            """
            pass

        def attach_async(self, _cb, jobId, current=None):
            """
            Returns the current omero.model.JobStatus for the
            Job id.
            Arguments:
            _cb -- The asynchronous callback object.
            jobId -- 
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if the omero.model.Job does not exist.
            """
            pass

        def getJob_async(self, _cb, current=None):
            """
            Returns the current omero.model.Job
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def jobStatus_async(self, _cb, current=None):
            """
            Returns the current omero.model.JobStatus. Will
            never return null.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def jobFinished_async(self, _cb, current=None):
            """
            Returns null if the omero.model.Job is
            not finished, otherwise the omero.RTime for when it
            completed.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def jobMessage_async(self, _cb, current=None):
            """
            Returns the current message for job. May be set during
            processing.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def jobRunning_async(self, _cb, current=None):
            """
            Returns true if the omero.model.Job is
            running, i.e. has an attached process.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def jobError_async(self, _cb, current=None):
            """
            Returns true if the omero.model.Job
            has thrown an error.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def cancelJob_async(self, _cb, current=None):
            """
            Marks a job for cancellation. Not every processor will
            check for the cancelled flag for a running job, but no
            non-running job will start if it has been cancelled.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def setStatus_async(self, _cb, status, current=None):
            """
            Updates the omero.model.JobStatus for the current
            job. The previous status is returned as a string. If the
            status is {@code CANCELLED}, this method is equivalent to
            {@code cancelJob}.
            Arguments:
            _cb -- The asynchronous callback object.
            status -- 
            current -- The Current object for the invocation.
            """
            pass

        def setMessage_async(self, _cb, message, current=None):
            """
            Sets the job's message string, and returns the previous
            value.
            Arguments:
            _cb -- The asynchronous callback object.
            message -- 
            current -- The Current object for the invocation.
            """
            pass

        def setStatusAndMessage_async(self, _cb, status, message, current=None):
            """
            Like {@code setStatus} but also sets the message.
            Arguments:
            _cb -- The asynchronous callback object.
            status -- 
            message -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_JobHandle)

        __repr__ = __str__

    _M_omero.api.JobHandlePrx = Ice.createTempClass()
    class JobHandlePrx(_M_omero.api.StatefulServiceInterfacePrx):

        """
        Submits a omero.model.Job and returns its database
        id. The only fields directly on status which are editable
        are message, scheduledFor and
        status. The latter two must be sensible.
        Arguments:
        job -- Not null
        _ctx -- The request context for the invocation.
        """
        def submit(self, job, _ctx=None):
            return _M_omero.api.JobHandle._op_submit.invoke(self, ((job, ), _ctx))

        """
        Submits a omero.model.Job and returns its database
        id. The only fields directly on status which are editable
        are message, scheduledFor and
        status. The latter two must be sensible.
        Arguments:
        job -- Not null
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_submit(self, job, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_submit.begin(self, ((job, ), _response, _ex, _sent, _ctx))

        """
        Submits a omero.model.Job and returns its database
        id. The only fields directly on status which are editable
        are message, scheduledFor and
        status. The latter two must be sensible.
        Arguments:
        job -- Not null
        """
        def end_submit(self, _r):
            return _M_omero.api.JobHandle._op_submit.end(self, _r)

        """
        Returns the current omero.model.JobStatus for the
        Job id.
        Arguments:
        jobId -- 
        _ctx -- The request context for the invocation.
        Throws:
        ApiUsageException -- if the omero.model.Job does not exist.
        """
        def attach(self, jobId, _ctx=None):
            return _M_omero.api.JobHandle._op_attach.invoke(self, ((jobId, ), _ctx))

        """
        Returns the current omero.model.JobStatus for the
        Job id.
        Arguments:
        jobId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_attach(self, jobId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_attach.begin(self, ((jobId, ), _response, _ex, _sent, _ctx))

        """
        Returns the current omero.model.JobStatus for the
        Job id.
        Arguments:
        jobId -- 
        Throws:
        ApiUsageException -- if the omero.model.Job does not exist.
        """
        def end_attach(self, _r):
            return _M_omero.api.JobHandle._op_attach.end(self, _r)

        """
        Returns the current omero.model.Job
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getJob(self, _ctx=None):
            return _M_omero.api.JobHandle._op_getJob.invoke(self, ((), _ctx))

        """
        Returns the current omero.model.Job
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getJob(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_getJob.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the current omero.model.Job
        Arguments:
        """
        def end_getJob(self, _r):
            return _M_omero.api.JobHandle._op_getJob.end(self, _r)

        """
        Returns the current omero.model.JobStatus. Will
        never return null.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def jobStatus(self, _ctx=None):
            return _M_omero.api.JobHandle._op_jobStatus.invoke(self, ((), _ctx))

        """
        Returns the current omero.model.JobStatus. Will
        never return null.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_jobStatus(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_jobStatus.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the current omero.model.JobStatus. Will
        never return null.
        Arguments:
        """
        def end_jobStatus(self, _r):
            return _M_omero.api.JobHandle._op_jobStatus.end(self, _r)

        """
        Returns null if the omero.model.Job is
        not finished, otherwise the omero.RTime for when it
        completed.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def jobFinished(self, _ctx=None):
            return _M_omero.api.JobHandle._op_jobFinished.invoke(self, ((), _ctx))

        """
        Returns null if the omero.model.Job is
        not finished, otherwise the omero.RTime for when it
        completed.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_jobFinished(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_jobFinished.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns null if the omero.model.Job is
        not finished, otherwise the omero.RTime for when it
        completed.
        Arguments:
        """
        def end_jobFinished(self, _r):
            return _M_omero.api.JobHandle._op_jobFinished.end(self, _r)

        """
        Returns the current message for job. May be set during
        processing.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def jobMessage(self, _ctx=None):
            return _M_omero.api.JobHandle._op_jobMessage.invoke(self, ((), _ctx))

        """
        Returns the current message for job. May be set during
        processing.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_jobMessage(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_jobMessage.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the current message for job. May be set during
        processing.
        Arguments:
        """
        def end_jobMessage(self, _r):
            return _M_omero.api.JobHandle._op_jobMessage.end(self, _r)

        """
        Returns true if the omero.model.Job is
        running, i.e. has an attached process.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def jobRunning(self, _ctx=None):
            return _M_omero.api.JobHandle._op_jobRunning.invoke(self, ((), _ctx))

        """
        Returns true if the omero.model.Job is
        running, i.e. has an attached process.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_jobRunning(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_jobRunning.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns true if the omero.model.Job is
        running, i.e. has an attached process.
        Arguments:
        """
        def end_jobRunning(self, _r):
            return _M_omero.api.JobHandle._op_jobRunning.end(self, _r)

        """
        Returns true if the omero.model.Job
        has thrown an error.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def jobError(self, _ctx=None):
            return _M_omero.api.JobHandle._op_jobError.invoke(self, ((), _ctx))

        """
        Returns true if the omero.model.Job
        has thrown an error.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_jobError(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_jobError.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns true if the omero.model.Job
        has thrown an error.
        Arguments:
        """
        def end_jobError(self, _r):
            return _M_omero.api.JobHandle._op_jobError.end(self, _r)

        """
        Marks a job for cancellation. Not every processor will
        check for the cancelled flag for a running job, but no
        non-running job will start if it has been cancelled.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def cancelJob(self, _ctx=None):
            return _M_omero.api.JobHandle._op_cancelJob.invoke(self, ((), _ctx))

        """
        Marks a job for cancellation. Not every processor will
        check for the cancelled flag for a running job, but no
        non-running job will start if it has been cancelled.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_cancelJob(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_cancelJob.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Marks a job for cancellation. Not every processor will
        check for the cancelled flag for a running job, but no
        non-running job will start if it has been cancelled.
        Arguments:
        """
        def end_cancelJob(self, _r):
            return _M_omero.api.JobHandle._op_cancelJob.end(self, _r)

        """
        Updates the omero.model.JobStatus for the current
        job. The previous status is returned as a string. If the
        status is {@code CANCELLED}, this method is equivalent to
        {@code cancelJob}.
        Arguments:
        status -- 
        _ctx -- The request context for the invocation.
        """
        def setStatus(self, status, _ctx=None):
            return _M_omero.api.JobHandle._op_setStatus.invoke(self, ((status, ), _ctx))

        """
        Updates the omero.model.JobStatus for the current
        job. The previous status is returned as a string. If the
        status is {@code CANCELLED}, this method is equivalent to
        {@code cancelJob}.
        Arguments:
        status -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setStatus(self, status, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_setStatus.begin(self, ((status, ), _response, _ex, _sent, _ctx))

        """
        Updates the omero.model.JobStatus for the current
        job. The previous status is returned as a string. If the
        status is {@code CANCELLED}, this method is equivalent to
        {@code cancelJob}.
        Arguments:
        status -- 
        """
        def end_setStatus(self, _r):
            return _M_omero.api.JobHandle._op_setStatus.end(self, _r)

        """
        Sets the job's message string, and returns the previous
        value.
        Arguments:
        message -- 
        _ctx -- The request context for the invocation.
        Returns: the previous message value
        """
        def setMessage(self, message, _ctx=None):
            return _M_omero.api.JobHandle._op_setMessage.invoke(self, ((message, ), _ctx))

        """
        Sets the job's message string, and returns the previous
        value.
        Arguments:
        message -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setMessage(self, message, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_setMessage.begin(self, ((message, ), _response, _ex, _sent, _ctx))

        """
        Sets the job's message string, and returns the previous
        value.
        Arguments:
        message -- 
        Returns: the previous message value
        """
        def end_setMessage(self, _r):
            return _M_omero.api.JobHandle._op_setMessage.end(self, _r)

        """
        Like {@code setStatus} but also sets the message.
        Arguments:
        status -- 
        message -- 
        _ctx -- The request context for the invocation.
        """
        def setStatusAndMessage(self, status, message, _ctx=None):
            return _M_omero.api.JobHandle._op_setStatusAndMessage.invoke(self, ((status, message), _ctx))

        """
        Like {@code setStatus} but also sets the message.
        Arguments:
        status -- 
        message -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setStatusAndMessage(self, status, message, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.JobHandle._op_setStatusAndMessage.begin(self, ((status, message), _response, _ex, _sent, _ctx))

        """
        Like {@code setStatus} but also sets the message.
        Arguments:
        status -- 
        message -- 
        """
        def end_setStatusAndMessage(self, _r):
            return _M_omero.api.JobHandle._op_setStatusAndMessage.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.JobHandlePrx.ice_checkedCast(proxy, '::omero::api::JobHandle', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.JobHandlePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::JobHandle'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_JobHandlePrx = IcePy.defineProxy('::omero::api::JobHandle', JobHandlePrx)

    _M_omero.api._t_JobHandle = IcePy.defineClass('::omero::api::JobHandle', JobHandle, -1, (), True, False, None, (_M_omero.api._t_StatefulServiceInterface,), ())
    JobHandle._ice_type = _M_omero.api._t_JobHandle

    JobHandle._op_submit = IcePy.Operation('submit', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.model._t_Job, False, 0),), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_attach = IcePy.Operation('attach', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.model._t_JobStatus, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_getJob = IcePy.Operation('getJob', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_omero.model._t_Job, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_jobStatus = IcePy.Operation('jobStatus', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_omero.model._t_JobStatus, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_jobFinished = IcePy.Operation('jobFinished', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_omero._t_RTime, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_jobMessage = IcePy.Operation('jobMessage', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_jobRunning = IcePy.Operation('jobRunning', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_jobError = IcePy.Operation('jobError', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_cancelJob = IcePy.Operation('cancelJob', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (), (), None, (_M_omero._t_ServerError,))
    JobHandle._op_setStatus = IcePy.Operation('setStatus', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_setMessage = IcePy.Operation('setMessage', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))
    JobHandle._op_setStatusAndMessage = IcePy.Operation('setStatusAndMessage', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero._t_RString, False, 0)), (), ((), IcePy._t_string, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.JobHandle = JobHandle
    del JobHandle

    _M_omero.api.JobHandlePrx = JobHandlePrx
    del JobHandlePrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `PyramidService.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ServicesF_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'ResolutionDescription' not in _M_omero.api.__dict__:
    _M_omero.api.ResolutionDescription = Ice.createTempClass()
    class ResolutionDescription(Ice.Object):
        def __init__(self, sizeX=0, sizeY=0):
            self.sizeX = sizeX
            self.sizeY = sizeY

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ResolutionDescription')

        def ice_id(self, current=None):
            return '::omero::api::ResolutionDescription'

        def ice_staticId():
            return '::omero::api::ResolutionDescription'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ResolutionDescription)

        __repr__ = __str__

    _M_omero.api.ResolutionDescriptionPrx = Ice.createTempClass()
    class ResolutionDescriptionPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ResolutionDescriptionPrx.ice_checkedCast(proxy, '::omero::api::ResolutionDescription', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ResolutionDescriptionPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ResolutionDescription'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ResolutionDescriptionPrx = IcePy.defineProxy('::omero::api::ResolutionDescription', ResolutionDescriptionPrx)

    _M_omero.api._t_ResolutionDescription = IcePy.defineClass('::omero::api::ResolutionDescription', ResolutionDescription, -1, (), False, False, None, (), (
        ('sizeX', (), IcePy._t_int, False, 0),
        ('sizeY', (), IcePy._t_int, False, 0)
    ))
    ResolutionDescription._ice_type = _M_omero.api._t_ResolutionDescription

    _M_omero.api.ResolutionDescription = ResolutionDescription
    del ResolutionDescription

    _M_omero.api.ResolutionDescriptionPrx = ResolutionDescriptionPrx
    del ResolutionDescriptionPrx

if '_t_ResolutionDescriptions' not in _M_omero.api.__dict__:
    _M_omero.api._t_ResolutionDescriptions = IcePy.defineSequence('::omero::api::ResolutionDescriptions', (), _M_omero.api._t_ResolutionDescription)

if 'PyramidService' not in _M_omero.api.__dict__:
    _M_omero.api.PyramidService = Ice.createTempClass()
    class PyramidService(_M_omero.api.StatefulServiceInterface):
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.PyramidService:
                raise RuntimeError('omero.api.PyramidService is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::PyramidService', '::omero::api::ServiceInterface', '::omero::api::StatefulServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::PyramidService'

        def ice_staticId():
            return '::omero::api::PyramidService'
        ice_staticId = staticmethod(ice_staticId)

        def requiresPixelsPyramid_async(self, _cb, current=None):
            """
            Whether or not this raw pixels store requires a backing
            pixels pyramid to provide sub-resolutions of the data.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getResolutionLevels_async(self, _cb, current=None):
            """
            Retrieves the number of resolution levels that the backing
            pixels pyramid contains.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getResolutionDescriptions_async(self, _cb, current=None):
            """
            Retrieves a more complete definition of the resolution
            level in question. The size of this array will be of
            length {@code getResolutionLevels}.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getResolutionLevel_async(self, _cb, current=None):
            """
            Retrieves the active resolution level.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def setResolutionLevel_async(self, _cb, resolutionLevel, current=None):
            """
            Sets the active resolution level.
            Arguments:
            _cb -- The asynchronous callback object.
            resolutionLevel -- The resolution level to be used by the pixel buffer.
            current -- The Current object for the invocation.
            """
            pass

        def getTileSize_async(self, _cb, current=None):
            """
            Retrieves the tile size for the pixel store.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_PyramidService)

        __repr__ = __str__

    _M_omero.api.PyramidServicePrx = Ice.createTempClass()
    class PyramidServicePrx(_M_omero.api.StatefulServiceInterfacePrx):

        """
        Whether or not this raw pixels store requires a backing
        pixels pyramid to provide sub-resolutions of the data.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: true if the pixels store requires a pixels pyramid and false otherwise.
        """
        def requiresPixelsPyramid(self, _ctx=None):
            return _M_omero.api.PyramidService._op_requiresPixelsPyramid.invoke(self, ((), _ctx))

        """
        Whether or not this raw pixels store requires a backing
        pixels pyramid to provide sub-resolutions of the data.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_requiresPixelsPyramid(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_requiresPixelsPyramid.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Whether or not this raw pixels store requires a backing
        pixels pyramid to provide sub-resolutions of the data.
        Arguments:
        Returns: true if the pixels store requires a pixels pyramid and false otherwise.
        """
        def end_requiresPixelsPyramid(self, _r):
            return _M_omero.api.PyramidService._op_requiresPixelsPyramid.end(self, _r)

        """
        Retrieves the number of resolution levels that the backing
        pixels pyramid contains.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: The number of resolution levels. This value does not necessarily indicate either the presence or absence of a pixels pyramid.
        """
        def getResolutionLevels(self, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionLevels.invoke(self, ((), _ctx))

        """
        Retrieves the number of resolution levels that the backing
        pixels pyramid contains.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getResolutionLevels(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionLevels.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the number of resolution levels that the backing
        pixels pyramid contains.
        Arguments:
        Returns: The number of resolution levels. This value does not necessarily indicate either the presence or absence of a pixels pyramid.
        """
        def end_getResolutionLevels(self, _r):
            return _M_omero.api.PyramidService._op_getResolutionLevels.end(self, _r)

        """
        Retrieves a more complete definition of the resolution
        level in question. The size of this array will be of
        length {@code getResolutionLevels}.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getResolutionDescriptions(self, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionDescriptions.invoke(self, ((), _ctx))

        """
        Retrieves a more complete definition of the resolution
        level in question. The size of this array will be of
        length {@code getResolutionLevels}.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getResolutionDescriptions(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionDescriptions.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves a more complete definition of the resolution
        level in question. The size of this array will be of
        length {@code getResolutionLevels}.
        Arguments:
        """
        def end_getResolutionDescriptions(self, _r):
            return _M_omero.api.PyramidService._op_getResolutionDescriptions.end(self, _r)

        """
        Retrieves the active resolution level.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: The active resolution level.
        """
        def getResolutionLevel(self, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionLevel.invoke(self, ((), _ctx))

        """
        Retrieves the active resolution level.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getResolutionLevel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_getResolutionLevel.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the active resolution level.
        Arguments:
        Returns: The active resolution level.
        """
        def end_getResolutionLevel(self, _r):
            return _M_omero.api.PyramidService._op_getResolutionLevel.end(self, _r)

        """
        Sets the active resolution level.
        Arguments:
        resolutionLevel -- The resolution level to be used by the pixel buffer.
        _ctx -- The request context for the invocation.
        """
        def setResolutionLevel(self, resolutionLevel, _ctx=None):
            return _M_omero.api.PyramidService._op_setResolutionLevel.invoke(self, ((resolutionLevel, ), _ctx))

        """
        Sets the active resolution level.
        Arguments:
        resolutionLevel -- The resolution level to be used by the pixel buffer.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setResolutionLevel(self, resolutionLevel, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_setResolutionLevel.begin(self, ((resolutionLevel, ), _response, _ex, _sent, _ctx))

        """
        Sets the active resolution level.
        Arguments:
        resolutionLevel -- The resolution level to be used by the pixel buffer.
        """
        def end_setResolutionLevel(self, _r):
            return _M_omero.api.PyramidService._op_setResolutionLevel.end(self, _r)

        """
        Retrieves the tile size for the pixel store.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: An array of length = 2 where the first value of the array is the tile width and the second value is the tile height.
        """
        def getTileSize(self, _ctx=None):
            return _M_omero.api.PyramidService._op_getTileSize.invoke(self, ((), _ctx))

        """
        Retrieves the tile size for the pixel store.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTileSize(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.PyramidService._op_getTileSize.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Retrieves the tile size for the pixel store.
        Arguments:
        Returns: An array of length = 2 where the first value of the array is the tile width and the second value is the tile height.
        """
        def end_getTileSize(self, _r):
            return _M_omero.api.PyramidService._op_getTileSize.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.PyramidServicePrx.ice_checkedCast(proxy, '::omero::api::PyramidService', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.PyramidServicePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::PyramidService'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_PyramidServicePrx = IcePy.defineProxy('::omero::api::PyramidService', PyramidServicePrx)

    _M_omero.api._t_PyramidService = IcePy.defineClass('::omero::api::PyramidService', PyramidService, -1, (), True, False, None, (_M_omero.api._t_StatefulServiceInterface,), ())
    PyramidService._ice_type = _M_omero.api._t_PyramidService

    PyramidService._op_requiresPixelsPyramid = IcePy.Operation('requiresPixelsPyramid', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    PyramidService._op_getResolutionLevels = IcePy.Operation('getResolutionLevels', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    PyramidService._op_getResolutionDescriptions = IcePy.Operation('getResolutionDescriptions', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_omero.api._t_ResolutionDescriptions, False, 0), (_M_omero._t_ServerError,))
    PyramidService._op_getResolutionLevel = IcePy.Operation('getResolutionLevel', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    PyramidService._op_setResolutionLevel = IcePy.Operation('setResolutionLevel', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0),), (), None, (_M_omero._t_ServerError,))
    PyramidService._op_getTileSize = IcePy.Operation('getTileSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), _M_Ice._t_IntSeq, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.PyramidService = PyramidService
    del PyramidService

    _M_omero.api.PyramidServicePrx = PyramidServicePrx
    del PyramidServicePrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



# Generated by slice2py - DO NOT EDIT!
#

import Ice
Ice.updateModule("omero.cmd")

# Modules:
import omero_cmd_API_ice
import omero_cmd_Admin_ice
import omero_cmd_Basic_ice
import omero_cmd_FS_ice
import omero_cmd_Graphs_ice
import omero_cmd_Mail_ice

# Submodules:
from . import graphs
